#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "nametab.h"
#include "comp.h"

name_table global_table, local_table;
name_table_ptr current_table = &global_table;

void init_name_tables() {
    lst_link(&global_table, &global_table);
    lst_link(&local_table, &local_table);
    current_table = &global_table;
}

void initialize_type(type_ptr t) {
    memset(t, 0, sizeof(t));
    lst_link(
        (block_ptr) &(t->arguments), (block_ptr) &(t->arguments)
    );
}

void erase_type(type_ptr t) {
    if (t->arguments.next != NULL)
        release_name_table((name_table_ptr) &(t->arguments));
    memset(t, 0, sizeof(t));
}

void copy_type(type_ptr dst, type_ptr src) {
    name_ptr srcarg;
    block_ptr dstarg;

    erase_type(dst);
    dst->base_type = src->base_type;
    dst->decl = src->decl;
    dst->base_type_size = src->base_type_size;
    dst->element_size = src->element_size;
    dst->dimension = src->dimension;
    dst->size = src->size;
    dst->num_arguments = src->num_arguments;

    /* copy a list of arguments */
    srcarg =  (name_ptr) src->arguments.next;
    dstarg = dst->arguments.next;
    while (srcarg != (name_ptr) &(src->arguments)) {
        name_ptr n = allocate_name_entry();
        copy_name_entry(n, srcarg);
        lst_addBefore(dstarg, (block_ptr) n);
        srcarg = (name_ptr) (srcarg->link.next);
    }
}

name_ptr allocate_name_entry() {
    name_ptr n = (name_ptr) malloc(sizeof(name_entry));
    memset(n, 0, sizeof(name_entry));
    initialize_type(&(n->typ));
    return n;
}

void release_name_entry(name_ptr n) {
    if (n == NULL) return;
    erase_type(&(n->typ));
    free(n);
}

void copy_name_entry(name_ptr dst, name_ptr src) {
    strcpy(dst->name, src->name);
    copy_type(&(dst->typ), &(src->typ));
    dst->offset = src->offset;
    dst->arg_number = src->arg_number;
}

void remove_name_entry(name_ptr n) {
    lst_remove(&(n->link));
    release_name_entry(n);
}

void add_name_entry(name_table_ptr t, name_ptr n) {
    lst_addBefore((block_ptr) t, (block_ptr) n); /* Add to the end of list */
}

void release_name_table(name_table_ptr t) { /* release all elements */
    block_ptr n = t->next;
    while (n != (block_ptr) t) {
        remove_name_entry((name_ptr) n);
        n = t->next;
    }
}

static int print_name_entry(block_ptr n, void *arg1, long arg2) {
    fprintf(
        dump_file,
        "symbol %s offset=%d type=%d decl=%d "
        "elem_size=%d dim=%d size=%d numargs=%d arg=%d\n",
        ((name_ptr) n)->name,
        ((name_ptr) n)->offset,
        ((name_ptr) n)->typ.base_type,
        ((name_ptr) n)->typ.decl,
        ((name_ptr) n)->typ.element_size,
        ((name_ptr) n)->typ.dimension,
        ((name_ptr) n)->typ.size,
        ((name_ptr) n)->typ.num_arguments,
        ((name_ptr) n)->arg_number
    );
    return 0;
}

void print_name_table(name_table_ptr t) {
    if (!(compiler_flags & DUMP_NAMETABLES))
        return;
    lst_loop(t, &print_name_entry, NULL, 0);
}

static int name_cmp(block_ptr n, void *name, long dummy) {
    if (strcmp(((name_ptr) n)->name, (const char *) name) == 0)
        return 1;
    return 0;
}

name_ptr name_lookup(const char *name, name_table_ptr t) {
    return (name_ptr) lst_loop(
        (block_ptr) t, &name_cmp, (void*) name, (long) 0
    );
}
