#ifndef TXTSTR_H
#define TXTSTR_H

class OutOfRangeException {
public:
    const char* reason;
    OutOfRangeException(): reason("") {}
    OutOfRangeException(const char* cause): reason(cause) {}
};

//
// TextString is the dynamic array of characters
//
class TextString {
    int     capacity;
    int     len;        // Not including the terminating zero character
    char*   str;
public:
    TextString();
    TextString(const TextString& line); // Copy constructor
    TextString(const char* line);
    ~TextString();

    // Assignment
    TextString& operator=(const TextString& line);
    TextString& operator=(const char* line);
    const char* getString() const { return str; }
    void setString(const char* line);
    void setString(const char* line, int length);

    // Character access
    char operator[](int i) const { return str[i]; }
    char& operator[](int i) { return str[i]; }
    char at(int i) const throw (OutOfRangeException) {
        if (i < 0 || i >= len)
            throw OutOfRangeException("Index out of range");
        return str[i];
    }
    char& at(int i) throw (OutOfRangeException) {
        if (i < 0 || i >= len)
            throw OutOfRangeException("Index out of range");
        return str[i];
    }

    // Covertion to C-style string
    operator const char*() const { return str; }

    int size() const { return len; }
    int length() const { return size(); }
    void setSize(int s);

    void truncate(int pos);
    void trim();    // Remove white space at the end of line

    void ensureCapacity(int n);

    void append(const char* line);
    void append(int character);
    TextString& operator+=(const char* line);
    TextString& operator+=(int character);
    TextString operator+(const char* line) const;
    TextString operator+(int character) const;
    void insert(int position, int character);
    void insert(int position, const char* line);
    void removeAt(int position);
};

#endif
